"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BootstrapAccountResolver = void 0;
/*
 * Copyright 2021 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const inquirer_1 = require("inquirer");
const symbol_sdk_1 = require("symbol-sdk");
const __1 = require("../");
class BootstrapAccountResolver {
    constructor(logger) {
        this.logger = logger;
    }
    async resolveAccount(networkType, account, keyName, nodeName, operationDescription, generateErrorMessage) {
        if (!account) {
            if (generateErrorMessage) {
                throw new __1.KnownError(generateErrorMessage);
            }
            this.logger.info(`Generating ${keyName} account...`);
            return symbol_sdk_1.Account.generateNewAccount(networkType);
        }
        if (!account.privateKey) {
            while (true) {
                this.logger.info('');
                this.logger.info(`${keyName} private key is required when ${operationDescription}.`);
                const address = symbol_sdk_1.PublicAccount.createFromPublicKey(account.publicKey, networkType).address.plain();
                const nodeDescription = nodeName === '' ? `of` : `of the Node's '${nodeName}'`;
                const responses = await inquirer_1.prompt([
                    {
                        name: 'value',
                        message: `Enter the 64 HEX private key ${nodeDescription} ${keyName} account with Address: ${address} and Public Key: ${account.publicKey}:`,
                        type: 'password',
                        mask: '*',
                        validate: __1.BootstrapUtils.isValidPrivateKey,
                    },
                ]);
                const privateKey = responses.value === '' ? undefined : responses.value.toUpperCase();
                if (!privateKey) {
                    this.logger.info('Please provide the private key.');
                }
                else {
                    const enteredAccount = symbol_sdk_1.Account.createFromPrivateKey(privateKey, networkType);
                    if (enteredAccount.publicKey.toUpperCase() !== account.publicKey.toUpperCase()) {
                        this.logger.info(`Invalid private key. Expected address is ${address} but you provided the private key for address ${enteredAccount.address.plain()}.\n`);
                        this.logger.info(`Please re-enter private key.`);
                    }
                    else {
                        account.privateKey = privateKey;
                        return symbol_sdk_1.Account.createFromPrivateKey(privateKey, networkType);
                    }
                }
            }
        }
        return symbol_sdk_1.Account.createFromPrivateKey(account.privateKey, networkType);
    }
    async shouldAnnounce(transaction, signedTransaction, transactionDescription) {
        return (await inquirer_1.prompt([
            {
                name: 'value',
                message: `Do you want to announce ${transactionDescription}?`,
                type: 'confirm',
                default: true,
            },
        ])).value;
    }
    async resolveCosigners(networkType, expectedAddresses, minApproval) {
        const providedAccounts = [];
        const allowedAddresses = [...expectedAddresses];
        while (true) {
            this.logger.info('');
            const expectedDescription = allowedAddresses.map((address) => address.plain()).join(', ');
            const responses = await inquirer_1.prompt([
                {
                    name: 'privateKey',
                    message: `Enter the 64 HEX private key of one of the addresses ${expectedDescription}. Already entered ${providedAccounts.length} out of ${minApproval} required cosigners.`,
                    type: 'password',
                    validate: __1.BootstrapUtils.isValidPrivateKey,
                },
            ]);
            const privateKey = responses.privateKey;
            if (!privateKey) {
                this.logger.info('Please provide the private key....');
            }
            else {
                const account = symbol_sdk_1.Account.createFromPrivateKey(privateKey, networkType);
                const expectedAddress = allowedAddresses.find((address) => address.equals(account.address));
                if (!expectedAddress) {
                    this.logger.info('');
                    this.logger.info(`Invalid private key. The entered private key has this ${account.address.plain()} address and it's not one of ${expectedDescription}. \n`);
                    this.logger.info(`Please re enter private key...`);
                }
                else {
                    allowedAddresses.splice(allowedAddresses.indexOf(expectedAddress), 1);
                    providedAccounts.push(account);
                    if (!allowedAddresses.length) {
                        this.logger.info('All cosigners have been entered.');
                        return providedAccounts;
                    }
                    if (providedAccounts.length == minApproval) {
                        this.logger.info(`Min Approval of ${minApproval} has been reached. Aggregate Complete transaction can be created.`);
                        return providedAccounts;
                    }
                    const { more } = await inquirer_1.prompt([
                        {
                            name: 'more',
                            message: `Do you want to enter more cosigners?`,
                            type: 'confirm',
                            default: providedAccounts.length < minApproval,
                        },
                    ]);
                    if (!more) {
                        return providedAccounts;
                    }
                    else {
                        this.logger.info('Please provide an additional private key....');
                    }
                }
            }
        }
    }
}
exports.BootstrapAccountResolver = BootstrapAccountResolver;
