"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BootstrapCommandUtils = void 0;
/*
 * Copyright 2021 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const command_1 = require("@oclif/command");
const figlet_1 = require("figlet");
const inquirer_1 = require("inquirer");
class BootstrapCommandUtils {
    static showBanner() {
        console.log(figlet_1.textSync('symbol-bootstrap', { horizontalLayout: 'fitted' }));
    }
    static getPasswordFlag(description) {
        return command_1.flags.string({
            description: description,
            parse(input) {
                const result = !input || BootstrapCommandUtils.isValidPassword(input);
                if (result === true)
                    return input;
                throw new Error(`--password is invalid, ${result}`);
            },
        });
    }
    static isValidPassword(input) {
        if (!input || input === '') {
            return true;
        }
        if (input.length >= 4)
            return true;
        return `Password must have at least 4 characters but got ${input.length}`;
    }
    static async resolvePassword(logger, providedPassword, noPassword, message, log) {
        if (!providedPassword) {
            if (noPassword) {
                if (log)
                    logger.warn(`Password has not been provided (--noPassword)! It's recommended to use one for security!`);
                return undefined;
            }
            const responses = await inquirer_1.prompt([
                {
                    name: 'password',
                    mask: '*',
                    message: message,
                    type: 'password',
                    validate: BootstrapCommandUtils.isValidPassword,
                },
            ]);
            if (responses.password === '' || !responses.password) {
                if (log)
                    logger.warn(`Password has not been provided (empty text)! It's recommended to use one for security!`);
                return undefined;
            }
            if (log)
                logger.info(`Password has been provided`);
            return responses.password;
        }
        if (log)
            logger.info(`Password has been provided`);
        return providedPassword;
    }
}
exports.BootstrapCommandUtils = BootstrapCommandUtils;
BootstrapCommandUtils.passwordPromptDefaultMessage = `Enter the password used to encrypt and decrypt custom presets, addresses.yml, and preset.yml files. When providing a password, private keys will be encrypted. Keep this password in a secure place!`;
BootstrapCommandUtils.helpFlag = command_1.flags.help({ char: 'h', description: 'It shows the help of this command.' });
BootstrapCommandUtils.targetFlag = command_1.flags.string({
    char: 't',
    description: 'The target folder where the symbol-bootstrap network is generated',
    default: 'target',
});
BootstrapCommandUtils.passwordFlag = BootstrapCommandUtils.getPasswordFlag(`A password used to encrypt and decrypt private keys in preset files like addresses.yml and preset.yml. Bootstrap prompts for a password by default, can be provided in the command line (--password=XXXX) or disabled in the command line (--noPassword).`);
BootstrapCommandUtils.noPasswordFlag = command_1.flags.boolean({
    description: 'When provided, Bootstrap will not use a password, so private keys will be stored in plain text. Use with caution.',
    default: false,
});
BootstrapCommandUtils.announceFlags = {
    password: BootstrapCommandUtils.passwordFlag,
    noPassword: BootstrapCommandUtils.noPasswordFlag,
    url: command_1.flags.string({
        char: 'u',
        description: 'the network url',
        default: 'http://localhost:3000',
    }),
    useKnownRestGateways: command_1.flags.boolean({
        description: 'Use the best NEM node available when announcing. Otherwise the command will use the node provided by the --url parameter.',
    }),
    ready: command_1.flags.boolean({
        description: 'If --ready is provided, the command will not ask for confirmation when announcing transactions.',
    }),
    maxFee: command_1.flags.integer({
        description: 'the max fee used when announcing (absolute). The node min multiplier will be used if it is not provided.',
    }),
    customPreset: command_1.flags.string({
        char: 'c',
        description: `This command uses the encrypted addresses.yml to resolve the main private key. If the main private is only stored in the custom preset, you can provide it using this param. Otherwise, the command may ask for it when required.`,
        required: false,
    }),
};
