"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigLoader = void 0;
/*
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const fs_1 = require("fs");
const _ = require("lodash");
const path_1 = require("path");
const symbol_sdk_1 = require("symbol-sdk");
const model_1 = require("../model");
const BootstrapUtils_1 = require("./BootstrapUtils");
const ConfigService_1 = require("./ConfigService");
const CryptoUtils_1 = require("./CryptoUtils");
class ConfigLoader {
    constructor(logger) {
        this.logger = logger;
    }
    async generateRandomConfiguration(oldAddresses, oldPresetData, presetData, accountResolver) {
        const networkType = presetData.networkType;
        const addresses = {
            version: this.getAddressesMigration(networkType).length + 1,
            networkType: networkType,
            nemesisGenerationHashSeed: presetData.nemesisGenerationHashSeed || (oldAddresses === null || oldAddresses === void 0 ? void 0 : oldAddresses.nemesisGenerationHashSeed) ||
                symbol_sdk_1.Convert.uint8ToHex(symbol_sdk_1.Crypto.randomBytes(32)),
            sinkAddress: presetData.sinkAddress || (oldAddresses === null || oldAddresses === void 0 ? void 0 : oldAddresses.sinkAddress),
        };
        //Sync address is generated on demand only
        const resolveSyncAddress = (providedAddress) => {
            if (providedAddress) {
                return symbol_sdk_1.Address.createFromRawAddress(providedAddress).plain();
            }
            addresses.sinkAddress = addresses.sinkAddress || symbol_sdk_1.Account.generateNewAccount(networkType).address.plain();
            return addresses.sinkAddress;
        };
        presetData.harvestNetworkFeeSinkAddress = resolveSyncAddress(presetData.harvestNetworkFeeSinkAddress);
        presetData.mosaicRentalFeeSinkAddress = resolveSyncAddress(presetData.mosaicRentalFeeSinkAddress);
        presetData.namespaceRentalFeeSinkAddress = resolveSyncAddress(presetData.namespaceRentalFeeSinkAddress);
        presetData.networkIdentifier = BootstrapUtils_1.BootstrapUtils.getNetworkIdentifier(networkType);
        presetData.networkName = BootstrapUtils_1.BootstrapUtils.getNetworkName(networkType);
        presetData.nemesisGenerationHashSeed = addresses.nemesisGenerationHashSeed;
        addresses.nodes = await this.generateNodeAccounts(accountResolver, oldAddresses, presetData, networkType);
        const shouldCreateNemesis = ConfigLoader.shouldCreateNemesis(presetData);
        if (shouldCreateNemesis) {
            const nemesisSigner = this.resolveNemesisAccount(presetData, oldAddresses);
            if (!nemesisSigner.privateKey) {
                throw new Error('Nemesis Signer Private Key should be resolved!');
            }
            addresses.nemesisSigner = nemesisSigner;
            presetData.nemesisSignerPublicKey = nemesisSigner.publicKey;
            presetData.nemesis.nemesisSignerPrivateKey = nemesisSigner.privateKey;
        }
        const nemesisSignerAddress = symbol_sdk_1.Address.createFromPublicKey(presetData.nemesisSignerPublicKey, networkType);
        if (!presetData.currencyMosaicId)
            presetData.currencyMosaicId = symbol_sdk_1.MosaicId.createFromNonce(symbol_sdk_1.MosaicNonce.createFromNumber(0), nemesisSignerAddress).toHex();
        if (!presetData.harvestingMosaicId) {
            if (!presetData.nemesis) {
                throw new Error('nemesis must be defined!');
            }
            if (presetData.nemesis.mosaics && presetData.nemesis.mosaics.length > 1) {
                presetData.harvestingMosaicId = symbol_sdk_1.MosaicId.createFromNonce(symbol_sdk_1.MosaicNonce.createFromNumber(1), nemesisSignerAddress).toHex();
            }
            else {
                presetData.harvestingMosaicId = presetData.currencyMosaicId;
            }
        }
        if (shouldCreateNemesis) {
            if (oldAddresses) {
                if (!oldPresetData) {
                    throw new Error('oldPresetData must be defined when upgrading!');
                }
                // Nemesis configuration cannot be changed on upgrade.
                addresses.mosaics = oldAddresses.mosaics;
                presetData.nemesis = oldPresetData.nemesis;
            }
            else {
                addresses.mosaics = this.processNemesisBalances(presetData, addresses, nemesisSignerAddress);
            }
        }
        return addresses;
    }
    resolveNemesisAccount(presetData, oldAddresses) {
        var _a, _b;
        const networkType = presetData.networkType;
        const signerPrivateKey = presetData.nemesis.nemesisSignerPrivateKey || ((_a = oldAddresses === null || oldAddresses === void 0 ? void 0 : oldAddresses.nemesisSigner) === null || _a === void 0 ? void 0 : _a.privateKey) ||
            symbol_sdk_1.Account.generateNewAccount(networkType).privateKey;
        const signerPublicKey = presetData.nemesisSignerPublicKey || ((_b = oldAddresses === null || oldAddresses === void 0 ? void 0 : oldAddresses.nemesisSigner) === null || _b === void 0 ? void 0 : _b.publicKey);
        const nemesisSigner = ConfigLoader.toConfigFromKeys(networkType, signerPublicKey, signerPrivateKey);
        if (!nemesisSigner) {
            throw new Error('Nemesis Signer should be resolved!');
        }
        return nemesisSigner;
    }
    sum(distribution, mosaicName) {
        return distribution
            .map((d, index) => {
            if (d.amount < 0) {
                throw new Error(`Nemesis distribution balance cannot be less than 0. Mosaic ${mosaicName}, distribution address: ${d.address}, amount: ${d.amount}, index ${index}. \nDistributions are:\n${BootstrapUtils_1.BootstrapUtils.toYaml(distribution)}`);
            }
            return d.amount;
        })
            .reduce((a, b) => a + b, 0);
    }
    processNemesisBalances(presetData, addresses, nemesisSignerAddress) {
        const privateKeySecurityMode = CryptoUtils_1.CryptoUtils.getPrivateKeySecurityMode(presetData.privateKeySecurityMode);
        const networkType = presetData.networkType;
        const mosaics = [];
        presetData.nemesis.mosaics.forEach((m, mosaicIndex) => {
            var _a;
            const accounts = this.generateAddresses(networkType, privateKeySecurityMode, m.accounts || 1);
            const id = symbol_sdk_1.MosaicId.createFromNonce(symbol_sdk_1.MosaicNonce.createFromNumber(mosaicIndex), nemesisSignerAddress).toHex();
            mosaics.push({
                id: id,
                name: m.name,
                accounts,
            });
            const getBalance = (nodeIndex) => {
                var _a, _b;
                const node = (_a = presetData === null || presetData === void 0 ? void 0 : presetData.nodes) === null || _a === void 0 ? void 0 : _a[nodeIndex];
                if (!node) {
                    return undefined;
                }
                const balance = (_b = node === null || node === void 0 ? void 0 : node.balances) === null || _b === void 0 ? void 0 : _b[mosaicIndex];
                if (balance !== undefined) {
                    return balance;
                }
                if (node.excludeFromNemesis) {
                    return 0;
                }
                return undefined;
            };
            const providedDistributions = [...(m.currencyDistributions || [])];
            (_a = addresses.nodes) === null || _a === void 0 ? void 0 : _a.forEach((node, index) => {
                const balance = getBalance(index);
                if (balance !== undefined)
                    providedDistributions.push({
                        address: node.main.address,
                        amount: balance,
                    });
            });
            const nodeMainAccounts = (addresses.nodes || []).filter((node, index) => node.main && getBalance(index) === undefined);
            const providedSupply = this.sum(providedDistributions, m.name);
            const remainingSupply = m.supply - providedSupply;
            const dynamicAccounts = accounts.length + nodeMainAccounts.length;
            const amountPerAccount = Math.floor(remainingSupply / dynamicAccounts);
            const maxHarvesterBalance = this.getMaxHarvesterBalance(presetData, mosaicIndex);
            const generatedAccounts = [
                ...accounts.map((a) => ({
                    address: a.address,
                    amount: amountPerAccount,
                })),
                ...nodeMainAccounts.map((n) => ({
                    address: n.main.address,
                    amount: Math.min(maxHarvesterBalance, amountPerAccount),
                })),
            ];
            m.currencyDistributions = [...generatedAccounts, ...providedDistributions].filter((d) => d.amount > 0);
            const generatedSupply = this.sum(generatedAccounts.slice(1), m.name);
            m.currencyDistributions[0].amount = m.supply - providedSupply - generatedSupply;
            const supplied = this.sum(m.currencyDistributions, m.name);
            if (m.supply != supplied) {
                throw new Error(`Invalid nemgen total supplied value, expected ${m.supply} but total is ${supplied}. \nDistributions are:\n${BootstrapUtils_1.BootstrapUtils.toYaml(m.currencyDistributions)}`);
            }
        });
        return mosaics;
    }
    getMaxHarvesterBalance(presetData, mosaicIndex) {
        return (presetData.nemesis.mosaics.length == 1 && mosaicIndex == 0) || (presetData.nemesis.mosaics.length > 1 && mosaicIndex == 1)
            ? presetData.maxHarvesterBalance
            : Number.MAX_SAFE_INTEGER;
    }
    static shouldCreateNemesis(presetData) {
        return (presetData.nemesis &&
            !presetData.nemesisSeedFolder &&
            (BootstrapUtils_1.BootstrapUtils.isYmlFile(presetData.preset) ||
                !fs_1.existsSync(path_1.join(BootstrapUtils_1.BootstrapUtils.DEFAULT_ROOT_FOLDER, 'presets', presetData.preset, 'seed'))));
    }
    generateAddresses(networkType, privateKeySecurityMode, accounts) {
        if (typeof accounts == 'number') {
            return ConfigLoader.getArray(accounts).map(() => ConfigLoader.toConfigFromAccount(symbol_sdk_1.Account.generateNewAccount(networkType)));
        }
        else {
            return accounts.map((key) => ConfigLoader.toConfigFromAccount(symbol_sdk_1.PublicAccount.createFromPublicKey(key, networkType)));
        }
    }
    static toAccountFromKeys(networkType, publicKey, privateKey) {
        if (privateKey) {
            const account = symbol_sdk_1.Account.createFromPrivateKey(privateKey, networkType);
            if (publicKey && account.publicKey.toUpperCase() != publicKey.toUpperCase()) {
                throw new Error('Invalid provided public key/private key!');
            }
            return account;
        }
        if (publicKey) {
            return symbol_sdk_1.PublicAccount.createFromPublicKey(publicKey, networkType);
        }
        return undefined;
    }
    static toConfigFromAccount(account) {
        // isntanceof doesn't work when loaded in multiple libraries.
        //https://stackoverflow.com/questions/59265098/instanceof-not-work-correctly-in-typescript-library-project
        if (account.constructor.name === symbol_sdk_1.Account.name) {
            return {
                privateKey: account.privateKey,
                publicKey: account.publicKey,
                address: account.address.plain(),
            };
        }
        return {
            publicKey: account.publicKey,
            address: account.address.plain(),
        };
    }
    static toConfigFromKeys(networkType, publicKey, privateKey) {
        const account = this.toAccountFromKeys(networkType, publicKey, privateKey);
        if (!account) {
            return undefined;
        }
        return this.toConfigFromAccount(account);
    }
    resolveGenerateErrorMessage(keyName, privateKeySecurityMode) {
        if (keyName === ConfigService_1.KeyName.Main &&
            (privateKeySecurityMode === model_1.PrivateKeySecurityMode.PROMPT_ALL ||
                privateKeySecurityMode === model_1.PrivateKeySecurityMode.PROMPT_MAIN_TRANSPORT ||
                privateKeySecurityMode === model_1.PrivateKeySecurityMode.PROMPT_MAIN)) {
            return `Account ${keyName} cannot be generated when Private Key Security Mode is ${privateKeySecurityMode}. Account won't be stored anywhere!. Please use ${model_1.PrivateKeySecurityMode.ENCRYPT}, or provider your ${keyName} account with custom presets!`;
        }
        if (keyName === ConfigService_1.KeyName.Transport &&
            (privateKeySecurityMode === model_1.PrivateKeySecurityMode.PROMPT_ALL ||
                privateKeySecurityMode === model_1.PrivateKeySecurityMode.PROMPT_MAIN_TRANSPORT)) {
            return `Account ${keyName} cannot be generated when Private Key Security Mode is ${privateKeySecurityMode}. Account won't be stored anywhere!. Please use ${model_1.PrivateKeySecurityMode.ENCRYPT}, ${model_1.PrivateKeySecurityMode.PROMPT_MAIN}, or provider your ${keyName} account with custom presets!`;
        }
        else {
            if (privateKeySecurityMode === model_1.PrivateKeySecurityMode.PROMPT_ALL) {
                return `Account ${keyName} cannot be generated when Private Key Security Mode is ${privateKeySecurityMode}. Account won't be stored anywhere! Please use ${model_1.PrivateKeySecurityMode.ENCRYPT}, ${model_1.PrivateKeySecurityMode.PROMPT_MAIN}, ${model_1.PrivateKeySecurityMode.PROMPT_MAIN_TRANSPORT}, or provider your ${keyName} account with custom presets!`;
            }
        }
        return undefined;
    }
    async generateAccount(accountResolver, networkType, privateKeySecurityMode, keyName, nodeName, oldStoredAccount, newProvidedAccount) {
        var _a, _b, _c;
        const oldAccount = ConfigLoader.toAccountFromKeys(networkType, oldStoredAccount === null || oldStoredAccount === void 0 ? void 0 : oldStoredAccount.publicKey.toUpperCase(), (_a = oldStoredAccount === null || oldStoredAccount === void 0 ? void 0 : oldStoredAccount.privateKey) === null || _a === void 0 ? void 0 : _a.toUpperCase());
        const newAccount = ConfigLoader.toAccountFromKeys(networkType, (_b = newProvidedAccount === null || newProvidedAccount === void 0 ? void 0 : newProvidedAccount.publicKey) === null || _b === void 0 ? void 0 : _b.toUpperCase(), (_c = newProvidedAccount === null || newProvidedAccount === void 0 ? void 0 : newProvidedAccount.privateKey) === null || _c === void 0 ? void 0 : _c.toUpperCase());
        const getAccountLog = (a) => `${keyName} Account ${a.address.plain()} Public Key ${a.publicKey} `;
        if (oldAccount && newAccount) {
            if (oldAccount.address.equals(newAccount.address)) {
                this.logger.info(`Reusing ${getAccountLog(newAccount)}`);
                return Object.assign(Object.assign({}, ConfigLoader.toConfigFromAccount(oldAccount)), ConfigLoader.toConfigFromAccount(newAccount));
            }
            this.logger.info(`Old ${getAccountLog(oldAccount)} has been changed. New ${getAccountLog(newAccount)} replaces it.`);
            return ConfigLoader.toConfigFromAccount(newAccount);
        }
        if (oldAccount) {
            this.logger.info(`Reusing ${getAccountLog(oldAccount)}...`);
            return ConfigLoader.toConfigFromAccount(oldAccount);
        }
        if (newAccount) {
            this.logger.info(`${getAccountLog(newAccount)} has been provided`);
            return ConfigLoader.toConfigFromAccount(newAccount);
        }
        const generateErrorMessage = this.resolveGenerateErrorMessage(keyName, privateKeySecurityMode);
        const account = await accountResolver.resolveAccount(networkType, newProvidedAccount || oldStoredAccount, keyName, nodeName, 'initialization', generateErrorMessage);
        return ConfigLoader.toConfigFromAccount(account);
    }
    async generateNodeAccount(accountResolver, oldNodeAccount, presetData, index, nodePreset, networkType) {
        const privateKeySecurityMode = CryptoUtils_1.CryptoUtils.getPrivateKeySecurityMode(presetData.privateKeySecurityMode);
        const name = nodePreset.name || `node-${index}`;
        const main = await this.generateAccount(accountResolver, networkType, privateKeySecurityMode, ConfigService_1.KeyName.Main, nodePreset.name, oldNodeAccount === null || oldNodeAccount === void 0 ? void 0 : oldNodeAccount.main, ConfigLoader.toConfigFromKeys(networkType, nodePreset.mainPublicKey, nodePreset.mainPrivateKey));
        const transport = await this.generateAccount(accountResolver, networkType, privateKeySecurityMode, ConfigService_1.KeyName.Transport, nodePreset.name, oldNodeAccount === null || oldNodeAccount === void 0 ? void 0 : oldNodeAccount.transport, ConfigLoader.toConfigFromKeys(networkType, nodePreset.transportPublicKey, nodePreset.transportPrivateKey));
        const friendlyName = nodePreset.friendlyName || main.publicKey.substr(0, 7);
        const nodeAccount = {
            name,
            friendlyName,
            roles: ConfigLoader.resolveRoles(nodePreset),
            main: main,
            transport: transport,
        };
        const useRemoteAccount = nodePreset.nodeUseRemoteAccount || presetData.nodeUseRemoteAccount;
        if (useRemoteAccount && (nodePreset.harvesting || nodePreset.voting))
            nodeAccount.remote = await this.generateAccount(accountResolver, networkType, privateKeySecurityMode, ConfigService_1.KeyName.Remote, nodePreset.name, oldNodeAccount === null || oldNodeAccount === void 0 ? void 0 : oldNodeAccount.remote, ConfigLoader.toConfigFromKeys(networkType, nodePreset.remotePublicKey, nodePreset.remotePrivateKey));
        if (nodePreset.harvesting)
            nodeAccount.vrf = await this.generateAccount(accountResolver, networkType, privateKeySecurityMode, ConfigService_1.KeyName.VRF, nodePreset.name, oldNodeAccount === null || oldNodeAccount === void 0 ? void 0 : oldNodeAccount.vrf, ConfigLoader.toConfigFromKeys(networkType, nodePreset.vrfPublicKey, nodePreset.vrfPrivateKey));
        if (nodePreset.rewardProgram)
            nodeAccount.agent = await this.generateAccount(accountResolver, networkType, privateKeySecurityMode, ConfigService_1.KeyName.Agent, nodePreset.name, oldNodeAccount === null || oldNodeAccount === void 0 ? void 0 : oldNodeAccount.agent, ConfigLoader.toConfigFromKeys(networkType, nodePreset.agentPublicKey, nodePreset.agentPrivateKey));
        return nodeAccount;
    }
    async generateNodeAccounts(accountResolver, oldAddresses, presetData, networkType) {
        return Promise.all((presetData.nodes || []).map((node, index) => { var _a; return this.generateNodeAccount(accountResolver, (_a = oldAddresses === null || oldAddresses === void 0 ? void 0 : oldAddresses.nodes) === null || _a === void 0 ? void 0 : _a[index], presetData, index, node, networkType); }));
    }
    static getArray(size) {
        return [...Array(size).keys()];
    }
    loadCustomPreset(customPreset, password) {
        if (!customPreset) {
            return {};
        }
        if (!fs_1.existsSync(customPreset)) {
            throw new BootstrapUtils_1.KnownError(`Custom preset '${customPreset}' doesn't exist. Have you provided the right --customPreset <customPrestFileLocation> ?`);
        }
        return BootstrapUtils_1.BootstrapUtils.loadYaml(customPreset, password);
    }
    static loadAssembly(preset, assembly, workingDir) {
        if (BootstrapUtils_1.BootstrapUtils.isYmlFile(assembly)) {
            const assemblyFile = BootstrapUtils_1.BootstrapUtils.resolveWorkingDirPath(workingDir, assembly);
            if (!fs_1.existsSync(assemblyFile)) {
                throw new BootstrapUtils_1.KnownError(`Assembly '${assembly}' does not exist. Have you provided the right --preset <preset> --assembly <assembly> ?`);
            }
            return BootstrapUtils_1.BootstrapUtils.loadYaml(assemblyFile, false);
        }
        const fileLocation = `${BootstrapUtils_1.BootstrapUtils.DEFAULT_ROOT_FOLDER}/presets/assemblies/assembly-${assembly}.yml`;
        if (fs_1.existsSync(fileLocation)) {
            return BootstrapUtils_1.BootstrapUtils.loadYaml(fileLocation, false);
        }
        throw new BootstrapUtils_1.KnownError(`Assembly '${assembly}' is not valid for preset '${preset}'. Have you provided the right --preset <preset> --assembly <assembly> ?`);
    }
    static loadNetworkPreset(preset, workingDir) {
        if (BootstrapUtils_1.BootstrapUtils.isYmlFile(preset)) {
            const presetFile = BootstrapUtils_1.BootstrapUtils.resolveWorkingDirPath(workingDir, preset);
            if (!fs_1.existsSync(presetFile)) {
                throw new BootstrapUtils_1.KnownError(`Preset '${presetFile}' does not exist. Have you provided the right --preset <preset>`);
            }
            return BootstrapUtils_1.BootstrapUtils.loadYaml(presetFile, false);
        }
        const bundledPreset = `${BootstrapUtils_1.BootstrapUtils.DEFAULT_ROOT_FOLDER}/presets/${preset}/network.yml`;
        if (!fs_1.existsSync(bundledPreset)) {
            throw new BootstrapUtils_1.KnownError(`Preset '${preset}' does not exist. Have you provided the right --preset <preset>`);
        }
        return BootstrapUtils_1.BootstrapUtils.loadYaml(bundledPreset, false);
    }
    static loadSharedPreset() {
        return BootstrapUtils_1.BootstrapUtils.loadYaml(path_1.join(BootstrapUtils_1.BootstrapUtils.DEFAULT_ROOT_FOLDER, 'presets', 'shared.yml'), false);
    }
    mergePresets(object, ...otherArgs) {
        var _a;
        const presets = [object, ...otherArgs];
        const reverse = [...presets].reverse();
        const inflation = ((_a = reverse.find((p) => p === null || p === void 0 ? void 0 : p.inflation)) === null || _a === void 0 ? void 0 : _a.inflation) || {};
        const presetData = _.merge({}, ...presets);
        if (!_.isEmpty(inflation))
            presetData.inflation = inflation;
        return presetData;
    }
    createPresetData(params) {
        var _a, _b, _c;
        const customPreset = params.customPreset;
        const customPresetObject = params.customPresetObject;
        const oldPresetData = params.oldPresetData;
        const customPresetFileObject = this.loadCustomPreset(customPreset, params.password);
        const preset = params.preset || ((_a = params.customPresetObject) === null || _a === void 0 ? void 0 : _a.preset) || (customPresetFileObject === null || customPresetFileObject === void 0 ? void 0 : customPresetFileObject.preset) || (oldPresetData === null || oldPresetData === void 0 ? void 0 : oldPresetData.preset);
        if (!preset) {
            throw new BootstrapUtils_1.KnownError('Preset value could not be resolved from target folder contents. Please provide the --preset parameter.');
        }
        const assembly = params.assembly || ((_b = params.customPresetObject) === null || _b === void 0 ? void 0 : _b.assembly) || (customPresetFileObject === null || customPresetFileObject === void 0 ? void 0 : customPresetFileObject.assembly) || ((_c = params.oldPresetData) === null || _c === void 0 ? void 0 : _c.assembly) ||
            ConfigService_1.Assembly.dual;
        const sharedPreset = ConfigLoader.loadSharedPreset();
        const networkPreset = ConfigLoader.loadNetworkPreset(preset, params.workingDir);
        const assemblyPreset = ConfigLoader.loadAssembly(preset, assembly, params.workingDir);
        const providedCustomPreset = this.mergePresets(customPresetFileObject, customPresetObject);
        const resolvedCustomPreset = _.isEmpty(providedCustomPreset) ? (oldPresetData === null || oldPresetData === void 0 ? void 0 : oldPresetData.customPresetCache) || {} : providedCustomPreset;
        const presetData = this.mergePresets(sharedPreset, networkPreset, assemblyPreset, resolvedCustomPreset);
        if (!ConfigLoader.presetInfoLogged) {
            this.logger.info(`Generating config from preset '${preset}'`);
            if (assembly) {
                this.logger.info(`Using assembly '${assembly}'`);
            }
            if (customPreset) {
                this.logger.info(`Using custom preset file '${customPreset}'`);
            }
        }
        if (!presetData.networkType) {
            throw new Error('Network Type could not be resolved. Have your provided the right --preset?');
        }
        ConfigLoader.presetInfoLogged = true;
        const presetDataWithDynamicDefaults = Object.assign(Object.assign({}, presetData), { version: 1, preset: preset, assembly: assembly || '', nodes: this.dynamicDefaultNodeConfiguration(presetData.nodes), customPresetCache: resolvedCustomPreset });
        return this.expandRepeat(presetDataWithDynamicDefaults);
    }
    dynamicDefaultNodeConfiguration(nodes) {
        return _.map(nodes || [], (node) => {
            return Object.assign(Object.assign({}, this.getDefaultConfiguration(node)), node);
        });
    }
    getDefaultConfiguration(node) {
        if (node.harvesting && node.api) {
            return {
                syncsource: true,
                filespooling: true,
                partialtransaction: true,
                addressextraction: true,
                mongo: true,
                zeromq: true,
                enableAutoSyncCleanup: false,
            };
        }
        if (node.api) {
            return {
                syncsource: false,
                filespooling: true,
                partialtransaction: true,
                addressextraction: true,
                mongo: true,
                zeromq: true,
                enableAutoSyncCleanup: false,
            };
        }
        // peer only (harvesting or not).
        return {
            syncsource: true,
            filespooling: false,
            partialtransaction: false,
            addressextraction: false,
            mongo: false,
            zeromq: false,
            enableAutoSyncCleanup: true,
        };
    }
    static resolveRoles(nodePreset) {
        if (nodePreset.roles) {
            return nodePreset.roles;
        }
        const roles = [];
        if (nodePreset.syncsource) {
            roles.push('Peer');
        }
        if (nodePreset.api) {
            roles.push('Api');
        }
        if (nodePreset.voting) {
            roles.push('Voting');
        }
        return roles.join(',');
    }
    expandRepeat(presetData) {
        return Object.assign(Object.assign({}, presetData), { databases: this.expandServicesRepeat(presetData, presetData.databases || []), nodes: this.expandServicesRepeat(presetData, presetData.nodes || []), gateways: this.expandServicesRepeat(presetData, presetData.gateways || []), explorers: this.expandServicesRepeat(presetData, presetData.explorers || []), wallets: this.expandServicesRepeat(presetData, presetData.wallets || []), faucets: this.expandServicesRepeat(presetData, presetData.faucets || []), nemesis: this.applyValueTemplate(presetData, presetData.nemesis) });
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    applyValueTemplate(context, value) {
        if (!value) {
            return value;
        }
        if (_.isArray(value)) {
            return this.expandServicesRepeat(context, value);
        }
        if (_.isObject(value)) {
            return _.mapValues(value, (v) => this.applyValueTemplate(Object.assign(Object.assign({}, context), value), v));
        }
        if (!_.isString(value)) {
            return value;
        }
        return BootstrapUtils_1.BootstrapUtils.runTemplate(value, context);
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    expandServicesRepeat(context, services) {
        return _.flatMap(services || [], (service) => {
            if (!_.isObject(service)) {
                return service;
            }
            const repeat = service.repeat;
            if (repeat === 0) {
                return [];
            }
            return _.range(repeat || 1).map((index) => {
                return _.omit(_.mapValues(service, (v) => this.applyValueTemplate(Object.assign(Object.assign(Object.assign({}, context), service), { $index: index }), v)), 'repeat');
            });
        });
    }
    loadExistingPresetDataIfPreset(target, password) {
        const generatedPresetLocation = this.getGeneratedPresetLocation(target);
        if (fs_1.existsSync(generatedPresetLocation)) {
            return BootstrapUtils_1.BootstrapUtils.loadYaml(generatedPresetLocation, password);
        }
        return undefined;
    }
    loadExistingPresetData(target, password) {
        const presetData = this.loadExistingPresetDataIfPreset(target, password);
        if (!presetData) {
            throw new Error(`The file ${this.getGeneratedPresetLocation(target)} doesn't exist. Have you executed the 'config' command? Have you provided the right --target param?`);
        }
        return presetData;
    }
    loadExistingAddressesIfPreset(target, password) {
        const generatedAddressLocation = this.getGeneratedAddressLocation(target);
        if (fs_1.existsSync(generatedAddressLocation)) {
            const presetData = this.loadExistingPresetData(target, password);
            return this.migrateAddresses(BootstrapUtils_1.BootstrapUtils.loadYaml(generatedAddressLocation, password), presetData.networkType);
        }
        return undefined;
    }
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    migrateAddresses(addresses, networkType) {
        const migrations = this.getAddressesMigration(networkType);
        return BootstrapUtils_1.BootstrapUtils.migrate(this.logger, 'addresses.yml', addresses, migrations);
    }
    getAddressesMigration(networkType) {
        return [
            {
                description: 'Key names migration',
                migrate(from) {
                    (from.nodes || []).forEach((nodeAddresses) => {
                        var _a, _b;
                        if (nodeAddresses.signing) {
                            nodeAddresses.main = nodeAddresses.signing;
                        }
                        else {
                            if (nodeAddresses.ssl) {
                                nodeAddresses.main = ConfigLoader.toConfigFromAccount(symbol_sdk_1.Account.createFromPrivateKey(nodeAddresses.ssl.privateKey, networkType));
                            }
                        }
                        nodeAddresses.transport = ConfigLoader.toConfigFromKeys(networkType, (_a = nodeAddresses === null || nodeAddresses === void 0 ? void 0 : nodeAddresses.node) === null || _a === void 0 ? void 0 : _a.publicKey, (_b = nodeAddresses === null || nodeAddresses === void 0 ? void 0 : nodeAddresses.node) === null || _b === void 0 ? void 0 : _b.privateKey);
                        if (!nodeAddresses.transport) {
                            nodeAddresses.transport = ConfigLoader.toConfigFromAccount(symbol_sdk_1.Account.generateNewAccount(networkType));
                        }
                        delete nodeAddresses.node;
                        delete nodeAddresses.signing;
                        delete nodeAddresses.ssl;
                    });
                    return from;
                },
            },
        ];
    }
    loadExistingAddresses(target, password) {
        const addresses = this.loadExistingAddressesIfPreset(target, password);
        if (!addresses) {
            throw new Error(`The file ${this.getGeneratedAddressLocation(target)} doesn't exist. Have you executed the 'config' command? Have you provided the right --target param?`);
        }
        return addresses;
    }
    getGeneratedPresetLocation(target) {
        return path_1.join(target, 'preset.yml');
    }
    getGeneratedAddressLocation(target) {
        return path_1.join(target, 'addresses.yml');
    }
}
exports.ConfigLoader = ConfigLoader;
ConfigLoader.presetInfoLogged = false;
